class Weather(object):
    def __init__(self):
        self.id = "WEATHER-"
        self.name = "WEATHER"
        self.color = "green"
        self.expiration = "no"
        self.location_name = ""
        self.country = ""
        self.lat = ""
        self.lon = ""
        self.icon = []
        self.temp = ""
        self.temp_unit = "Celsius"
        self.temp_max = ""
        self.temp_min = ""
        self.pressure = ""
        self.pressure_unit = "hPa"
        self.humidity = ""
        self.humidity_unit = "%"
        self.clouds = ""
        self.clouds_unit = "%"
        self.sunrise = ""
        self.sunset = ""
        self.wind_speed = ""
        self.wind_speed_unit = "meter/sec"
        self.last_time_update = ""
        self.last_date_update = ""
        self.rules = []

    def device_mapping(self, device):
        self.id = device["id"]
        self.name = device["name"]




class ProfileDto(object):
    def __init__(self):
        self.email = ""
        self.name = ""
        self.surname = ""
        self.password = ""
        self.user_id = ""

    def constructor(self, email, name, surname, password):
        self.email = email
        self.name = name
        self.surname = surname
        self.password = password

    def constructor_map(self, profile_map):
        if "email" in profile_map.keys():
            self.email = profile_map["email"]
        if "name" in profile_map.keys():
            self.name = profile_map["name"]
        if "surname" in profile_map.keys():
            self.surname = profile_map["surname"]
        if "password" in profile_map.keys():
            self.password = profile_map["password"]






class TimerAntecedent(object):
    def __init__(self):
        self.device_id = ""
        self.device_name = ""
        self.measure_time = ""
        self.measure_day = ""
        self.condition_time = "between"
        self.condition_day = "="
        self.day_start_value = []
        self.time_start_value = ""
        self.time_stop_value = ""
        self.evaluation = "false"
        self.check_time = "true"
        self.check_date = "true"

    def antecedent_mapping(self, antecedent):
        self.device_id = antecedent["device_id"]
        self.device_name = antecedent["device_name"]
        self.condition_time = antecedent["condition_time"]
        self.condition_day = antecedent["condition_day"]
        self.check_time = antecedent["check_time"]
        self.check_date = antecedent["check_date"]
        self.day_start_value = antecedent["day_start_value"]
        self.time_start_value = antecedent["time_start_value"]
        self.time_stop_value = antecedent["time_stop_value"]



from ..WeatherAntecedentDTO import IconAntecedent


class IconAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.icon_codes = {
            "01d": 1,  # clear sky
            "02d": 2,  # few clouds
            "03d": 3,  # scattered clouds
            "04d": 4,  # broken clouds
            "09d": 5,  # shower rain
            "10d": 6,  # rain
            "11d": 7,  # thunderstorm
            "13d": 8,  # snow
            "50d": 9  # mist
        }

    def get_icon_antecedent(self, user_id, rule_id, device_id):
        dto = IconAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto.check_icon = self.r.get(key_pattern + ":check_icon")
        dto.icon_condition = self.r.get(key_pattern + ":icon_condition")
        dto.icon_start_value = self.r.get(key_pattern + ":icon_start_value")
        dto.icon_stop_value = self.r.get(key_pattern + ":icon_stop_value")
        dto.icon_evaluation = self.r.get(key_pattern + ":icon_evaluation")
        location_name = self.r.get("user:" + user_id + ":location:name")
        dto.icon_value = list(self.r.smembers("weather:" + location_name + ":icon"))
        return dto

    def set_icon_antecedent(self, user_id, rule_id, device_id, dto: IconAntecedent):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_icon", dto.check_icon)
        self.r.set(key_pattern + ":icon_condition", dto.icon_condition)
        self.r.set(key_pattern + ":icon_start_value", dto.icon_start_value)
        self.r.set(key_pattern + ":icon_stop_value", dto.icon_stop_value)
        self.r.set(key_pattern + ":icon_evaluation", dto.icon_evaluation)

    def delete_icon_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.delete(key_pattern + ":check_icon")
        self.r.delete(key_pattern + ":icon_condition")
        self.r.delete(key_pattern + ":icon_start_value")
        self.r.delete(key_pattern + ":icon_stop_value")
        self.r.delete(key_pattern + ":icon_evaluation")

    def register_icon_antecedent(self, user_id, rule_id, device_id):
        dto = IconAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_icon", dto.check_icon)
        self.r.set(key_pattern + ":icon_condition", dto.icon_condition)
        self.r.set(key_pattern + ":icon_start_value", dto.icon_start_value)
        self.r.set(key_pattern + ":icon_stop_value", dto.icon_stop_value)
        self.r.set(key_pattern + ":icon_evaluation", dto.icon_evaluation)

    def icon_antecedent_evaluation(self, user_id, rule_id, device_id):
        try:
            dto = self.get_icon_antecedent(user_id, rule_id, device_id)
            evaluation = "true"
            if dto.check_icon == "true":
                evaluation = "false"
                icon_value = self.icon_codes[dto.icon_value]
                start_value = self.icon_codes[dto.icon_start_value]
                stop_value = self.icon_codes[dto.icon_stop_value]
                if dto.icon_condition == "between":
                    if start_value < icon_value <= stop_value:
                        evaluation = "true"
                elif dto.icon_condition == ">":
                    if icon_value > start_value:
                        evaluation = "true"
                elif dto.icon_condition == "<":
                    if icon_value < stop_value:
                        evaluation = "true"
                if dto.icon_evaluation != evaluation:
                    key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                    self.r.set(key_pattern + ":icon_evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"




class SwitchConsequent(object):
    def __init__(self):
        self.device_id = ""
        self.device_name = ""
        self.if_value = "on"
        self.else_value = "off"
        self.delay = "0"
        self.delay_unit_measure = "seconds"
        self.order = ""
        self.automatic = "true"
        self.measure = "off"

    def consequent_mapping(self, consequent):
        self.device_id = consequent["device_id"]
        self.device_name = consequent["device_name"]
        self.delay = consequent["delay"]
        self.order = consequent["order"]



from .SwitchConsequentDTO import SwitchConsequent
from .SwitchAntecedentFunctions import SwitchAntecedentFunction


class SwitchConsequentFunction(object):
    def __init__(self, redis):
        self.r = redis
        self.switch_antecedent_functions = SwitchAntecedentFunction(redis)

    def get_consequent(self, user_id, rule_id, device_id):
        try:
            dto = SwitchConsequent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            dto.device_id = device_id
            dto.device_name = self.r.get("device:" + device_id + ":name")
            dto.delay = self.r.get(key_pattern + ":delay")
            dto.order = self.r.get(key_pattern + ":order")
            dto.automatic = self.r.get("device:" + device_id + ":automatic")
            if dto.automatic == "true":
                dto.measure = self.r.get("device:" + device_id + ":measure")
            else:
                dto.measure = "manual"
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def get_consequent_slim(self, user_id, rule_id, device_id):
        try:
            dto = SwitchConsequent()
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            dto.device_id = device_id
            dto.device_name = self.r.get("device:" + device_id + ":name")
            dto.order = self.r.get(key_pattern + ":order")
            dto.delay = self.r.get(key_pattern + ":delay")
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_consequent(self, user_id, rule_id, device_id):
        try:
            device_antecedents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_antecedents")
            if device_id in device_antecedents:
                self.switch_antecedent_functions.delete_antecedent(user_id, rule_id, device_id)
            self.r.lrem("device:" + device_id + ":rules", rule_id)
            self.r.lrem("user:" + user_id + ":rule:" + rule_id + ":device_consequents", device_id)
            key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
            self.r.delete(key_pattern + ":delay")
            self.r.delete(key_pattern + ":order")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def add_consequent(self, user_id, rule_id, device_id):
        try:
            device_consequents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_consequents")
            result = "false"
            if device_id not in device_consequents:
                consequent = SwitchConsequent()
                consequent.order = str(len(device_consequents))
                self.r.rpush("device:" + device_id + ":rules", rule_id)
                self.r.rpush("user:" + user_id + ":rule:" + rule_id + ":device_consequents", device_id)
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + device_id
                self.r.set(key_pattern + ":delay", consequent.delay)
                self.r.set(key_pattern + ":order", consequent.order)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def update_consequent(self, user_id, rule_id, new_consequent):
        try:
            consequent = SwitchConsequent()
            consequent.consequent_mapping(new_consequent)
            device_consequents = self.r.lrange("user:" + user_id + ":rule:" + rule_id + ":device_consequents")
            result = "false"
            if consequent.device_id in device_consequents:
                key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_consequents:" + consequent.device_id
                self.r.set(key_pattern + ":delay", consequent.delay)
                self.r.set(key_pattern + ":order", consequent.order)
                result = "true"
            return result
        except Exception as error:
            print(repr(error))
            return "error"

    def switch_evaluation(self, user_id, device_id):
        key_pattern = "device:" + device_id
        automatic = self.r.get(key_pattern + ":automatic")
        output = "false"
        if automatic == "true" and self.r.exists(key_pattern + ":measure") == 1:
            rules = self.r.lrange(key_pattern + ":rules")
            new_status = "off"
            current_status = self.r.get("device:" + device_id + ":measure")
            for rule in rules:
                rule_evaluation = self.r.get("user:" + user_id + ":rule:" + rule + ":evaluation")
                if rule_evaluation == "true":
                    new_status = "on"
                    break
            if current_status != new_status:
                output = new_status
        return output



class Rule(object):
    def __init__(self):
        self.id = ""
        self.name = ""
        self.last_time_on = "-"
        self.last_time_off = "-"
        self.last_date_on = "-"
        self.last_date_off = "-"
        self.device_antecedents = []
        self.device_consequents = []
        self.rule_antecedents = []
        self.rule_consequents = []
        self.evaluation = "false"

    def json_mapping(self, rule_json):
        self.id = rule_json["id"]
        self.name = rule_json["name"]
        self.device_antecedents = rule_json["device_antecedents"]
        self.device_consequents = rule_json["device_consequents"]
        self.rule_antecedents = rule_json["rule_antecedents"]
        self.rule_consequents = rule_json["rule_consequents"]



from ..WeatherAntecedentDTO import HumidityAntecedent


class HumidityAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_humidity_antecedent(self, user_id, rule_id, device_id):
        dto = HumidityAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto.check_humidity = self.r.get(key_pattern + ":check_humidity")
        dto.humidity_condition = self.r.get(key_pattern + ":humidity_condition")
        dto.humidity_start_value = self.r.get(key_pattern + ":humidity_start_value")
        dto.humidity_stop_value = self.r.get(key_pattern + ":humidity_stop_value")
        dto.humidity_evaluation = self.r.get(key_pattern + ":humidity_evaluation")
        location_name = self.r.get("user:" + user_id + ":location:name")
        dto.humidity_value = self.r.get("weather:" + location_name + ":humidity")
        return dto

    def set_humidity_antecedent(self, user_id, rule_id, device_id, dto: HumidityAntecedent):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_humidity", dto.check_humidity)
        self.r.set(key_pattern + ":humidity_condition", dto.humidity_condition)
        self.r.set(key_pattern + ":humidity_start_value", dto.humidity_start_value)
        self.r.set(key_pattern + ":humidity_stop_value", dto.humidity_stop_value)
        self.r.set(key_pattern + ":humidity_evaluation", dto.humidity_evaluation)

    def delete_humidity_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.delete(key_pattern + ":check_humidity")
        self.r.delete(key_pattern + ":humidity_condition")
        self.r.delete(key_pattern + ":humidity_start_value")
        self.r.delete(key_pattern + ":humidity_stop_value")
        self.r.delete(key_pattern + ":humidity_evaluation")

    def register_humidity_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto = HumidityAntecedent()
        self.r.set(key_pattern + ":check_humidity", dto.check_humidity)
        self.r.set(key_pattern + ":humidity_condition", dto.humidity_condition)
        self.r.set(key_pattern + ":humidity_start_value", dto.humidity_start_value)
        self.r.set(key_pattern + ":humidity_stop_value", dto.humidity_stop_value)
        self.r.set(key_pattern + ":humidity_evaluation", dto.humidity_evaluation)

    def humidity_antecedent_evaluation(self, user_id, rule_id, device_id):
        try:
            dto = self.get_humidity_antecedent(user_id, rule_id, device_id)
            evaluation = "true"
            if dto.check_humidity == "true":
                evaluation = "false"
                if dto.humidity_condition == "between":
                    if dto.humidity_start_value < dto.humidity_value <= dto.humidity_stop_value:
                        evaluation = "true"
                elif dto.humidity_condition == ">":
                    if dto.humidity_value > dto.humidity_start_value:
                        evaluation = "true"
                elif dto.humidity_condition == "<":
                    if dto.humidity_value < dto.humidity_stop_value:
                        evaluation = "true"
                if dto.humidity_evaluation != evaluation:
                    key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                    self.r.set(key_pattern + ":humidity_evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"
